function [S0vector,v0,figure1] = MM_inhib(varargin)
% Usage [S0,vs,figh] = MM_inhib(Ivector,Km,KI,KIprime,Vmax,S0vector);
%   or just MM_inhib to use defaults
% Jason Kahn U Maryland 2/25/2016  version 1.01
% All arguments are optional but must be given in order
% Graphs v0 vs. [S] for each [I], and also the Lineweaver-Burke plots
% Returns S0vector, the matrix of v0 vs. S for each [I],and the figure
% handle in case you want to export it or close it.
% Assumes initial rate conditions, so no point showing a time course.
% Assumes [E] << Km and KI and KIprime so can't titrate out I
% In the workspace, can define S0vector as for example
% >> S0 = [0.05 0.07 0.1 0.3 0.5 0.75 1 1.25 1.5 2 3.5 5 7 10];
% and Ivector as >> Ivector = [0 1e-6 3e-6 1e-5 5e-5 1e-4 ];
% Defaults to MM_inhib(Ivector,2e-5,1e-5,100,6e-3,S0);
% Change the S0vector (in units of Km) to explore different ranges of [S]
% Set defaults:
Ivector = [0 1e-6 3e-6 1e-5 5e-5 1e-4 ];  % Molar
Km = 2e-5;  % Molar
KI = 1e-5;  % Molar; infinite for pure uncompetitive inhibition
KIprime = 100; % Molar; infinite for pure competitive inhibition
Vmax = 6e-3; % Ms-1. The number is arbitrary as everything scales to it.
S0 = [0.05 0.07 0.1 0.3 0.5 0.75 1 1.25 1.5 2 3.5 5 7 10]; % in units of Km
% 
optargs = {Ivector Km KI KIprime Vmax S0};
numopts = 6;
% Change default values if user provides them
numvarargs = length(varargin);
if numvarargs > numopts
    error('MM_inhib:TooManyInputs', ...
        'MM_inhib allows at most %d optional inputs',numopts);
end

% now put these defaults into the valuesToUse cell array, 
% and overwrite the ones specified in varargin.
optargs(1:numvarargs) = varargin;

% Place optional args back into in memorable variable names
[Ivector,Km,KI,KIprime,Vmax,S0] = optargs{:};
S0vector = S0'*Km;
% exvec gives the range for extrapolation of the L-B plot to see the
% intersection point.
% Calculate the intersection point [which is 1/[S] = -KI/(KIprime*Km)] and 
% then show it as long as it is not too far to the left, always go at least to -2/Km.
leftedge = max(-50/Km,min(-1.5*KI/(KIprime*Km),-2/Km));
exvec = [leftedge 0.5*leftedge 0 1/max(S0vector)]';

%exvec = [-2/Km -1/Km 0 1/max(S0vector)]';
% Should not need to change anything below
alpha = 1 + Ivector/KI;
alphaprime = 1 + Ivector/KIprime;
num_S = size(S0vector,1);
v0 = zeros(num_S,length(Ivector));
for i = 1:length(Ivector)
    % Here's the guts of the program: calculating v0 according to the MM
    % equation
    v0(:,i) = (1/alphaprime(i))*Vmax*S0vector./((alpha(i)/alphaprime(i))*Km + S0vector);
end
figure1 = figure('Position',[10 10 800 350]);
subplot(1,2,1)
plot(S0vector,v0,'s-',[0 min(S0vector)]',[zeros(size(v0(1,:))) ; v0(1,:)],'k')
ax1 = axis;
axis([ax1(1) ax1(2) ax1(3) max(ax1(4),Vmax)])
title('Inhibited M-M kinetics','FontSize',14,'FontWeight','bold')
ylabel('Initial rate v0','FontSize',14,'FontWeight','bold')
xlabel('[Substrate]','FontSize',14,'FontWeight','bold')
% String array for the legend
s = cell(1,length(Ivector));
for k=1:length(Ivector)
  s{k} = ['[I] = ', num2str(Ivector(k))];
end
% Create legend for the selected plots
legend(s,'FontSize',12,'Location','east');
% Create string for text box
str = {['Km = ',num2str(Km),' M'],['KI = ',num2str(KI), ' M'],['KIp = ',num2str(KIprime), ' M'],...
    ['Vmax = ',num2str(Vmax), ' M-1s-1'],['Max [I] = ',num2str(max(Ivector)), ' M']};
text(0.8*(max(S0vector)),Vmax*0.2,str,'FontSize',12,'EdgeColor','b','BackGroundColor',[1 1 1])
subplot(1,2,2)
extrap = interp1(1./S0vector,1./v0,exvec,'linear','extrap');
plot(1./S0vector,1./v0,'o-')
title('Lineweaver-Burke (zoomed in)','FontSize',14,'FontWeight','bold')
ylabel('1/v0','FontSize',14,'FontWeight','bold')
xlabel('1/[Substrate]','FontSize',14,'FontWeight','bold')
% String array for the legend
s = cell(1,length(Ivector));
for k=1:length(Ivector)
  s{k} = ['[I] = ', num2str(Ivector(k))];
end
% Create legend for the selected plots
legend(s,'FontSize',12,'Location','northwest');
hold on
plot(exvec,extrap,'k')
ax = axis;
% This is where we zoom in so we can see the L-B plot lines better; may
% need improvement
axis([min(exvec) ax(2)/2 min(0,min(min(extrap))) ax(4)/2])
xL = xlim;
yL = ylim;
line([0 0], yL);  %x-axis
line(xL, [0 0]);  %y-axis
end
