% This is the MMgraph_scr.m script for illustrating progressively more
%   realistic enzymatic reactions.
% Jason Kahn U Maryland 2/25/2016  version 1.01
%  The top row of plots is the overall reaction, 
%   the bottom is initial rates. It starts with pure ideal M-M kinetics,
%   then looks at the effects of product inhibition, then  reversible reactions.
% All assuming steady state MM kinetics at small [E] so E does not titrate out S or P
% This is a script rather than a function to allow fiddling with all inputs
%   as desired. Calls RevMicMen(S0,P0,KmS,KmP,Keq,VmaxS,Tf), which
%   integrates under steady-state conditions. 
% Print nice output with e.g. export_fig -m3 ~/Desktop/mmkin2.png if you
% have downloaded the nice export_fig from the File Exchange.
% The figure handle is returned in fig1 so export_fig(fig1, filename);
% Note this is very kludgy and options are changed throughout below!
KmS = 1e-3;
VmaxS = 1.5e-3;  % VmaxP is determined from Km's and Keq
KmP = 100; % i.e. infinitely weak to start, changes to newKmP in Condition 2
newKmP = 1e-3;
tmax = 80;  % Reasonable range will depend on Vmax
tshort = 4;
Keq = 1e8; % Start with irreversible rxn, changes to newKeq in Condition 3
newKeq = 2;
% Vector of substrate concentrations
S0vec = [1e-4 3e-4 1e-3 3e-3 1e-2 3e-2];  % can only do 6 concentrations
%
fig1 = figure;
pos_fig1 = [0 0 1200 800];
set(fig1,'Position',pos_fig1)
%
% Condition 1: Straight up pure unidirectional MM
%
[ t1,concs1 ] = RevMicMen(S0vec(1),0,KmS,KmP,Keq,VmaxS,tmax);
[ t2,concs2 ] = RevMicMen(S0vec(2),0,KmS,KmP,Keq,VmaxS,tmax);
[ t3,concs3 ] = RevMicMen(S0vec(3),0,KmS,KmP,Keq,VmaxS,tmax);
[ t4,concs4 ] = RevMicMen(S0vec(4),0,KmS,KmP,Keq,VmaxS,tmax);
[ t5,concs5 ] = RevMicMen(S0vec(5),0,KmS,KmP,Keq,VmaxS,tmax);
[ t6,concs6 ] = RevMicMen(S0vec(6),0,KmS,KmP,Keq,VmaxS,tmax);
subplot(2,3,1)
plot(t6,concs6(:,2),t5,concs5(:,2),t4,concs4(:,2),...
    t3,concs3(:,2),t2,concs2(:,2),t1,concs1(:,2),'Linewidth',2)
set(gca,'FontSize',12)
axis([0 tmax 0 max(S0vec)])
title({'Irreversible Rxn, No product binding',''},'FontSize',14)
ylabel('[Product]','FontSize',12)
xlabel({'','Time'},'FontSize',12)
legstr = {['[S] = ' num2str(S0vec(6))],...
     ['[S] = ' num2str(S0vec(5))],...
     ['[S] = ' num2str(S0vec(4))],...
     ['[S] = ' num2str(S0vec(3))],...
     ['[S] = ' num2str(S0vec(2))],...
     ['[S] = ' num2str(S0vec(1))]};
%legend(legstr,'Location','northwest','FontSize',12,'EdgeColor','b')
legend1 = legend(legstr,'FontSize',12,'EdgeColor','b','LineWidth',1.5);
set(legend1,...
    'Position',[0.14 0.77 0.1 0.14])
% would be nice if there were an easier way to adjust legend line spacing
%
% Condition 1, short tines: Same as 1, but focus on short times to illustrate v0 going to Vmax
%   Recalculate curves to get more points at shorter t
%
[ t1,concs1 ] = RevMicMen(S0vec(1),0,KmS,KmP,Keq,VmaxS,tshort);
[ t2,concs2 ] = RevMicMen(S0vec(2),0,KmS,KmP,Keq,VmaxS,tshort);
[ t3,concs3 ] = RevMicMen(S0vec(3),0,KmS,KmP,Keq,VmaxS,tshort);
[ t4,concs4 ] = RevMicMen(S0vec(4),0,KmS,KmP,Keq,VmaxS,tshort);
[ t5,concs5 ] = RevMicMen(S0vec(5),0,KmS,KmP,Keq,VmaxS,tshort);
[ t6,concs6 ] = RevMicMen(S0vec(6),0,KmS,KmP,Keq,VmaxS,tshort);
%figure
subplot(2,3,4)
plot(t6,concs6(:,2),t5,concs5(:,2),t4,concs4(:,2),...
    t3,concs3(:,2),t2,concs2(:,2),t1,concs1(:,2),'Linewidth',2)
set(gca,'FontSize',12)
axis([0 tshort 0 max(S0vec)/6])
title({'Irrev Rxn, No product binding, short t',''},'FontSize',14)
ylabel('[Product]','FontSize',12)
xlabel({'','Time'},'FontSize',12)
legstr = {['KmS = ' num2str(KmS)],'',['VmaxS = ',num2str(VmaxS)],'',['KmP = ' num2str(KmP)]};
%legend(legstr,'Location','northwest','FontSize',12)
text(0.1*tshort,0.8*max(S0vec)/6,legstr,'FontSize',12,'EdgeColor','b')
%
% Condition 2: Effect of product inhibition, even in the absence of net back rxn
%
%newKmP = 1e-3;  % tight binding of product -- change above
%Keq = 1e8;   % so it is still irrelevant (done above now)
% tmax = 5000; % May need to uncomment to change tmax
%
[ t1,concs1 ] = RevMicMen(S0vec(1),0,KmS,newKmP,Keq,VmaxS,tmax);
[ t2,concs2 ] = RevMicMen(S0vec(2),0,KmS,newKmP,Keq,VmaxS,tmax);
[ t3,concs3 ] = RevMicMen(S0vec(3),0,KmS,newKmP,Keq,VmaxS,tmax);
[ t4,concs4 ] = RevMicMen(S0vec(4),0,KmS,newKmP,Keq,VmaxS,tmax);
[ t5,concs5 ] = RevMicMen(S0vec(5),0,KmS,newKmP,Keq,VmaxS,tmax);
[ t6,concs6 ] = RevMicMen(S0vec(6),0,KmS,newKmP,Keq,VmaxS,tmax);
%figure
subplot(2,3,2)
plot(t6,concs6(:,2),t5,concs5(:,2),t4,concs4(:,2),...
    t3,concs3(:,2),t2,concs2(:,2),t1,concs1(:,2),'Linewidth',2)
set(gca,'FontSize',12)
axis([0 tmax 0 max(S0vec)])
title({'Irreversible Rxn w/ Product Inhibition',''},'FontSize',14)
ylabel('[Product]','FontSize',12)
xlabel({'','Time'},'FontSize',12)
legstr = {['New KmP = ' num2str(newKmP)],'',['Keq = ',num2str(Keq)]};
%legend(legstr,'Location','northwest','FontSize',12)
text(0.1*tmax,0.8*max(S0vec),legstr,'FontSize',12,'EdgeColor','b')
%
% Effect of product inhibition, even in the absence of net back rxn, short t
%
[ t1,concs1 ] = RevMicMen(S0vec(1),0,KmS,newKmP,Keq,VmaxS,tshort);
[ t2,concs2 ] = RevMicMen(S0vec(2),0,KmS,newKmP,Keq,VmaxS,tshort);
[ t3,concs3 ] = RevMicMen(S0vec(3),0,KmS,newKmP,Keq,VmaxS,tshort);
[ t4,concs4 ] = RevMicMen(S0vec(4),0,KmS,newKmP,Keq,VmaxS,tshort);
[ t5,concs5 ] = RevMicMen(S0vec(5),0,KmS,newKmP,Keq,VmaxS,tshort);
[ t6,concs6 ] = RevMicMen(S0vec(6),0,KmS,newKmP,Keq,VmaxS,tshort);
%figure
subplot(2,3,5)
plot(t6,concs6(:,2),t5,concs5(:,2),t4,concs4(:,2),...
    t3,concs3(:,2),t2,concs2(:,2),t1,concs1(:,2),'Linewidth',2)
set(gca,'FontSize',12)
axis([0 tshort 0 max(S0vec)/6])
title({'Irrev Rxn w/ Product Inhibition, Short t',''},'FontSize',14)
ylabel('[Product]','FontSize',12)
xlabel({'','Time'},'FontSize',12)
%
% Condition 3: Effect of reversible rxn -- change newKeq as desired
%
%newKeq = 5; % Now changed above
%
[ t1,concs1 ] = RevMicMen(S0vec(1),0,KmS,newKmP,newKeq,VmaxS,tmax);
[ t2,concs2 ] = RevMicMen(S0vec(2),0,KmS,newKmP,newKeq,VmaxS,tmax);
[ t3,concs3 ] = RevMicMen(S0vec(3),0,KmS,newKmP,newKeq,VmaxS,tmax);
[ t4,concs4 ] = RevMicMen(S0vec(4),0,KmS,newKmP,newKeq,VmaxS,tmax);
[ t5,concs5 ] = RevMicMen(S0vec(5),0,KmS,newKmP,newKeq,VmaxS,tmax);
[ t6,concs6 ] = RevMicMen(S0vec(6),0,KmS,newKmP,newKeq,VmaxS,tmax);
%figure
subplot(2,3,3)
plot(t6,concs6(:,2),t5,concs5(:,2),t4,concs4(:,2),...
    t3,concs3(:,2),t2,concs2(:,2),t1,concs1(:,2),'Linewidth',2)
set(gca,'FontSize',12)
axis([0 tmax 0 max(S0vec)])
title({['Reversible Reaction, Keq = ',num2str(newKeq)],''},'FontSize',14)
ylabel('[Product]','FontSize',12)
xlabel({'','Time'},'FontSize',12)
legstr = {['KmP = ' num2str(newKmP)],'',['New Keq = ',num2str(newKeq)]};
%legend(legstr,'Location','northwest','FontSize',12)
text(0.1*tmax,0.8*max(S0vec),legstr,'FontSize',12,'EdgeColor','b')
%
% Effect of reversible rxn, short t
%
[ t1,concs1 ] = RevMicMen(S0vec(1),0,KmS,newKmP,newKeq,VmaxS,tshort);
[ t2,concs2 ] = RevMicMen(S0vec(2),0,KmS,newKmP,newKeq,VmaxS,tshort);
[ t3,concs3 ] = RevMicMen(S0vec(3),0,KmS,newKmP,newKeq,VmaxS,tshort);
[ t4,concs4 ] = RevMicMen(S0vec(4),0,KmS,newKmP,newKeq,VmaxS,tshort);
[ t5,concs5 ] = RevMicMen(S0vec(5),0,KmS,newKmP,newKeq,VmaxS,tshort);
[ t6,concs6 ] = RevMicMen(S0vec(6),0,KmS,newKmP,newKeq,VmaxS,tshort);
%figure
subplot(2,3,6)
plot(t6,concs6(:,2),t5,concs5(:,2),t4,concs4(:,2),...
    t3,concs3(:,2),t2,concs2(:,2),t1,concs1(:,2),'Linewidth',2)
set(gca,'FontSize',12)
axis([0 4 0 5e-3])
title({'Reversible Reaction, Short t',''},'FontSize',14)
ylabel('[Product]','FontSize',12)
xlabel({'','Time'},'FontSize',12)


