function [T,Y] = MMkinetics(varargin)
% Function to simulate Michaelis-Menten enzyme kinetics as a function of time,
%   assuming irreversible conversion of S to P and no product inhibition
% Jason Kahn U Maryland 2/25/2016  version 1.01
% Plots results in 6 plots comparing steady-state and pre-steady state time scales
% Arguments (all optional) must be in the following order: k1,kr1,kcat,S0,E0,t_end,savefig flag
% You can leave arguments off at the end but no skipping.
% With no arguments defaults to: MMkinetics(5e6,6e4,300,0.1,1e-5,74.7,0);
% Returns an array of times and []'s of Efree, ES, S, and P
% Calls function MMkin to provide differential rates and to regenerate rates for plotting purposes
% Calls function export_fig if and only if savefig flag is set to 1
%   Download export_fig from http://www.mathworks.com/matlabcentral/fileexchange/23629-exportfig
%
% Define rate constants and initial concentrations
% Defaults:
k1 = 5e6; % M-1s-1
kr1 = 6e4; % s-1
S0 = 100e-3; % M
E0 = 1e-5; % M
kcat = 300; % s-1
Km = (kr1+kcat)/k1 ; % M
Vmax = kcat*E0; % Ms-1
v0 = Vmax*S0/(S0 + Km); % Ms-1
t_end = 2*S0/v0; % s Roughly 
%
% Default is not to write files. Set savefig = 0 to prevent saving
% Set savefig to 1 to activate saving images
savefig = 0;
optargs = {k1 kr1 kcat S0 E0 t_end savefig};
numopts = 7;
% Change default values if user provides them
numvarargs = length(varargin);
if numvarargs > numopts
    error('MMkinetics:TooManyInputs', ...
        'MMkinetics allows at most %d optional inputs',numopts);
end

% now put these defaults into the valuesToUse cell array, 
% and overwrite the ones specified in varargin.
optargs(1:numvarargs) = varargin;

% Place optional args back into in memorable variable names
[k1,kr1,kcat,S0,E0,t_end,savefig] = optargs{:};
Km = (kr1+kcat)/k1 ; % M
Vmax = kcat*E0; % Ms-1
v0 = Vmax*S0/(S0 + Km); % Ms-1
t1 = 2*(1/k1 + 1/(kr1+kcat)); % s 
if numvarargs < 6
    t_end = 2*S0/v0; % s Roughly 
end
disp(['k1 = ',num2str(k1),' M-1s-1'])
disp(['kr1 = ',num2str(kr1), ' s-1'])
disp(['kcat = ',num2str(kcat), ' s-1'])
disp(['Substrate at t0 = ',num2str(S0), ' M'])
disp(['Enzyme at t0 = ',num2str(E0), ' M'])
disp(['Km = ',num2str(Km), ' M'])
disp(['Vmax = ',num2str(Vmax), ' Ms-1'])
disp(['v0 = ',num2str(v0), ' Ms-1'])
disp(['End time is ',num2str(t_end), ' s'])
disp(['Save figure flag is ',num2str(savefig)])
concs0 = [E0 0 S0 0];  % concentration of Efree, ES, S, and P
% pnum is for numbering plots sequentially. 
% Limit number of files exported to lim_files
% savefig = 1 means we are saving files
if savefig == 1;
    lim_files = 50;
    fileout = 'MMKinetics_';
    pnum = 1;
    while (pnum <= lim_files)
        fntest = [fileout,num2str(pnum),'.jpg'];
        if (exist(fntest,'file'))
            pnum = pnum+1;
        else
            disp(['Exported file is ',fntest])
            break
        end
    end
    if (pnum == lim_files+1)
        disp(['Exceeded file count: overwriting ',fntest])
    end
end
% Change Rscale to optimize appearance; should not be necessary
 Rscale = 2.0;
%---------------------------
% Should not have to touch anything below here

% Integrate differential equation to give time course. This is the
% point of the whole exercise. Integrate pre-steady state and long-time
% reaction separately. The ode15s solver is better for stiff differential
% equations.
options = odeset('RelTol',1e-5,'AbsTol',[1e-7 1e-7 1e-4 1e-4],'NonNegative',[1 2 3 4]);
[T1,Y1] = ode15s(@(t,concs) MMkin(t,concs,k1,kr1,kcat),[0 t1],concs0',options);
%concs0;
midconcs = Y1(end,:);

options = odeset('RelTol',1e-6,'AbsTol',[1e-9 1e-9 1e-6 1e-6],'NonNegative',[1 2 3 4]);
[T2,Y2] = ode15s(@(t,concs) MMkin(t,concs,k1,kr1,kcat),[t1 t_end],midconcs,options);
%lastconcs = Y2(end,:);

% % Instantaneous compression at t1_end and allow to evolve until t2_end
% [T2,Y2] = ode45(@(t,concs) TwoAtoB(t,concs,k1,kr1),[t1_end t2_end],Vratio1*lastconcs,options);
% lastconcs = Y2(end,:);
% 
% % Add (or remove) A at t2_end, evolve to t3_end
% [T3,Y3] = ode45(@(t,concs) TwoAtoB(t,concs,k1,kr1),[t2_end t3_end],[lastconcs(1)+deltaA lastconcs(2)],options);
% lastconcs = Y3(end,:);
% 
% % Add (or remove) B at t3_enf, evolve to t4_end
% [T4,Y4] = ode45(@(t,concs) TwoAtoB(t,concs,k1,kr1),[t3_end t4_end],[lastconcs(1) lastconcs(2)+deltaB],options);
% lastconcs = Y4(end,:);
% 
% % Instantaneous compression at t4_end and allow to evolve until t5_end
% [T5,Y5] = ode45(@(t,concs) TwoAtoB(t,concs,k1,kr1),[t4_end t5_end],Vratio2*lastconcs,options);
% %lastconcs = Y5(end,:);

% Collect time and concentration arrays

T = [T1 ; T2 ];
Y = [Y1 ; Y2 ];
% Error check below should never be triggered because we specified
% NonNegative above
if (min(min(Y)) < 0)
    disp('NOTE:')
    disp('    Negative concentration detected: results are likely WRONG!   ')
end
% recalculate instantaneous rates
numt = length(T);
rates = zeros(numt,4);
%Prod_rate = zeros(numt,1);
for i=1:numt
    rates(i,:) = MMkin(1,Y(i,:),k1,kr1,kcat);
end
Prod_rate = rates(:,4);
%
numt1 = length(T1);
rates1 = zeros(numt1,4);
%Prod_rate = zeros(numt,1);
for i=1:numt1
    rates1(i,:) = MMkin(1,Y(i,:),k1,kr1,kcat);
end
Prod_rate1 = rates1(:,4);

% Plot results
set(0,'defaultAxesFontName', 'Arial')
set(0,'defaultAxesFontSize', 14)
set(0,'defaultAxesFontWeight', 'bold')

%scrsz = get(0,'ScreenSize');
figure1 = figure('Position',[10 10 1024 768]);
% S and P
sub1=subplot(3,2,1);
plot(T,Y(:,3),'b',T,Y(:,4),'g','LineWidth',2)
leg1 = legend('Substrate','Product');
set(leg1,'FontSize',14);
title('Michaelis-Menten Kinetics','FontSize',14,'FontWeight','bold')
ylabel('S and P Concentrations','FontSize',14,'FontWeight','bold')
%xlabel('Time (s)','FontSize',14,'FontWeight','bold')
ylim(sub1,[0 max(max([Y(:,3) Y(:,4)]))*1.3])
t1 = text(min(T)+0.03*(max(T)-min(T)),max(max([Y(:,3) Y(:,4)]))*1.1,'Rxn: E + S <-> ES -> E + P','FontSize',14,'FontWeight','bold');
set(t1,'FontName','Arial')
% E and ES
sub3 = subplot(3,2,3);
plot(T,Y(:,1),'r',T,Y(:,2),'k','LineWidth',2)
leg1 = legend('Free enzyme E','ES Complex');
set(leg1,'FontSize',14);
title('Enzyme Forms','FontSize',14,'FontWeight','bold')
ylabel('E and ES Concentrations','FontSize',14,'FontWeight','bold')
%xlabel('Time (s)','FontSize',14,'FontWeight','bold')
% t1 = text(min(T)+0.03*(max(T)-min(T)),max(max([Y(:,1) Y(:,2)]))*1.1,'Rxn: E + S <-> ES -> E + P','FontSize',14,'FontWeight','bold');
% set(t1,'FontName','Arial')
%
sub5 = subplot(3,2,5);
%plot(T,(rates(:,1)),'-g',T,(rates(:,2)),'-.k','LineWidth',2)
plot(T,Prod_rate,'-c','LineWidth',2)
% ylim should be adaptive depending on rate constants and concentrations
% to show the rates to best advantage
ylim(sub5,[0 max(median(Prod_rate),Rscale*max(Prod_rate))]);
ylabel('Rate (M-1s-1)','FontSize',14,'FontWeight','bold')
xlabel('Time (s)','FontSize',14,'FontWeight','bold')
title('Rate of Product Formation','FontSize',14,'FontWeight','bold')
%
% Same plots at very short times
sub2 = subplot(3,2,2);
plot(T1,Y1(:,3),'b',T1,Y1(:,4),'g','LineWidth',2)
%leg1 = legend('Substrate','Product');
%set(leg1,'FontSize',14);
title('Pre-steady State Kinetics','FontSize',14,'FontWeight','bold')
ylabel('S and P Concentrations','FontSize',14,'FontWeight','bold')
%xlabel('Time (s)','FontSize',14,'FontWeight','bold')
ylim(sub2,[0 max(max([Y(:,3) Y(:,4)]))*1.3])
t1 = text(min(T1)+0.03*(max(T1)-min(T1)),max(max([Y1(:,3) Y1(:,4)]))*1.1,'Note very short times','FontSize',14,'FontWeight','bold');
set(t1,'FontName','Arial')
% Skip this out if exporting -- there's a bug in Matlab or Export_fig that
% prevents it from saving correctly...hmmmm seems to work now.
%if (savefig == 0)
str = {['k1 = ',num2str(k1),' M-1s-1'],['kr1 = ',num2str(kr1), ' s-1'],['kcat = ',num2str(kcat), ' s-1'],...
    ['Substrate at t0 = ',num2str(S0), ' M'],['Enzyme at t0 = ',num2str(E0), ' M'],['Km = ',num2str(Km), ' M'],...
    ['Vmax = ',num2str(Vmax), ' Ms-1'],['v0 = ',num2str(v0), ' Ms-1'],['End time is ',num2str(t_end), ' s'],...
    ['Save figure flag is ',num2str(savefig)]};
annotation(figure1,'textbox', [.745 .72 .148 .165],'BackGroundColor',[1 1 1],...
           'String', str,'FitBoxToText','off');
%end
%'FitBoxToText','off'
sub4 = subplot(3,2,4);
plot(T1,Y1(:,1),'r',T1,Y1(:,2),'k','LineWidth',2)
leg1 = legend('Free enzyme E','ES Complex');
set(leg1,'FontSize',14);
title('Establishment of E<->ES Steady State','FontSize',14,'FontWeight','bold')
ylabel('E and ES Concentrations','FontSize',14,'FontWeight','bold')
%xlabel('Time (s)','FontSize',14,'FontWeight','bold')
%t1 = text(min(T1)+0.03*(max(T1)-min(T1)),max(max([Y1(:,1) Y1(:,2)]))*1.1,'Rxn: E + S <-> ES -> E + P','FontSize',14,'FontWeight','bold');
%set(t1,'FontName','Arial')
%
sub6 = subplot(3,2,6);
%plot(T,(rates(:,1)),'-g',T,(rates(:,2)),'-.k','LineWidth',2)
plot(T1,Prod_rate1,'-c','LineWidth',2)
% ylim should be adaptive to show the rates to best advantage
ylim(sub6,[0 max(median(Prod_rate),Rscale*max(Prod_rate))]);
ylabel('Rate (M-1s-1)','FontSize',14,'FontWeight','bold')
title('Rate of Product Formation','FontSize',14,'FontWeight','bold')
xlabel('Early Times (s)','FontSize',14,'FontWeight','bold')

% Export jpeg or png. Does not work yet with annotation textbox above
% (actually now it does seem to work).
if (savefig == 1)
    export_fig (fntest)   % or (fntest,'-png')
end
 


